@@ -62,8 +62,16 @@ func printSetting(entry syscfg.CfgEntry) {
 		util.StatusMessage(util.VERBOSITY_DEFAULT,
 			"    * Overridden: ")
 		for i := 1; i < len(entry.History); i++ {
-			util.StatusMessage(util.VERBOSITY_DEFAULT, "%s, ",
-				entry.History[i].Source.FullName())
+			var fullName string
+
+			lpkg := entry.History[i].Source
+			if lpkg == nil {
+				fullName = "<command line>"
+			} else {
+				fullName = lpkg.FullName()
+			}
+
+			util.StatusMessage(util.VERBOSITY_DEFAULT, "%s, ", fullName)
 		}
 		util.StatusMessage(util.VERBOSITY_DEFAULT,
 			"default=%s\n", entry.History[0].Value)
@@ -82,8 +90,16 @@ func printBriefSetting(entry syscfg.CfgEntry) {
 	var extras []string
 
 	if len(entry.History) > 1 {
-		s := fmt.Sprintf("overridden by %s",
-			entry.History[len(entry.History)-1].Source.FullName())
+		var fullName string
+
+		lpkg := entry.History[len(entry.History)-1].Source
+		if lpkg == nil {
+			fullName = "<command line>"
+		} else {
+			fullName = lpkg.FullName()
+		}
+
+		s := fmt.Sprintf("overridden by %s", fullName)
 		extras = append(extras, s)
 	}
 	if len(entry.ValueRefName) > 0 {
@@ -467,8 +483,17 @@ func printStage(sf stage.StageFunc) {
 	util.StatusMessage(util.VERBOSITY_DEFAULT, "%s:\n", sf.Name)
 	util.StatusMessage(util.VERBOSITY_DEFAULT, "    Package: %s\n",
 		sf.Pkg.FullName())
-	util.StatusMessage(util.VERBOSITY_DEFAULT, "    Stage:  %s\n",
-		valSettingString(sf.Stage))
+	if len(sf.Stage.Afters) > 0 || len(sf.Stage.Befores) > 0 {
+		for _, s := range sf.Stage.Afters {
+			util.StatusMessage(util.VERBOSITY_DEFAULT, "    After:   %s\n", s)
+		}
+		for _, s := range sf.Stage.Befores {
+			util.StatusMessage(util.VERBOSITY_DEFAULT, "    Before:  %s\n", s)
+		}
+	} else {
+		util.StatusMessage(util.VERBOSITY_DEFAULT, "    Stage:   %s\n",
+			valSettingString(sf.Stage))
+	}
 }
 
 func printStageBriefOne(sf stage.StageFunc,
@@ -550,6 +575,30 @@ func printSysinitBrief(targetName string, scfg sysinit.SysinitCfg) {
 	printStageBriefTable(scfg.StageFuncs)
 }
 
+func printSysinitGraphviz(targetName string, scfg sysinit.SysinitCfg) {
+	if errText := scfg.ErrorText(); errText != "" {
+		util.StatusMessage(util.VERBOSITY_DEFAULT, "!!! %s\n\n", errText)
+	}
+
+	fmt.Printf("digraph sysinit {\n")
+	for _, sf := range scfg.StageFuncs {
+		if len(sf.Stage.Afters) == 0 && len(sf.Stage.Befores) == 0 {
+			stage, _ := sf.Stage.IntVal()
+			fmt.Printf("  %s [label=\"%s (%d)\"];\n", sf.Name, sf.Name, stage)
+		}
+		for _, depSf := range sf.DepsI {
+			fmt.Printf("  %s -> %s;\n", sf.Name, depSf.Name)
+		}
+		for _, depStr := range sf.Stage.Afters {
+			fmt.Printf("  %s -> %s [label=\"$after:%s\"];\n", depStr, sf.Name, depStr)
+		}
+		for _, depStr := range sf.Stage.Befores {
+			fmt.Printf("  %s -> %s [label=\"$before:%s\"];\n", sf.Name, depStr, depStr)
+		}
+	}
+	fmt.Printf("}\n")
+}
+
 func targetSysinitShowCmd(cmd *cobra.Command, args []string) {
 	if len(args) < 1 {
 		NewtUsage(cmd,
@@ -596,6 +645,29 @@ func targetSysinitBriefCmd(cmd *cobra.Command, args []string) {
 	}
 }
 
+func targetSysinitGraphvizCmd(cmd *cobra.Command, args []string) {
+	if len(args) < 1 {
+		NewtUsage(cmd,
+			util.NewNewtError("Must specify target or unittest name"))
+	}
+
+	TryGetProject()
+
+	for i, arg := range args {
+		b, err := TargetBuilderForTargetOrUnittest(arg)
+		if err != nil {
+			NewtUsage(cmd, err)
+		}
+
+		res := targetBuilderConfigResolve(b)
+		printSysinitGraphviz(b.GetTarget().Name(), res.SysinitCfg)
+
+		if i < len(args)-1 {
+			util.StatusMessage(util.VERBOSITY_DEFAULT, "\n")
+		}
+	}
+}
+
 func printSysdownCfg(targetName string, scfg sysdown.SysdownCfg) {
 	util.StatusMessage(util.VERBOSITY_DEFAULT, "Sysdown config for %s:\n",
 		targetName)
@@ -786,6 +858,9 @@ func targetCfgCmdAll() []*cobra.Command {
 		Run:   targetConfigShowCmd,
 	}
 
+	configShowCmd.Flags().StringVarP(&util.InjectSyscfg, "syscfg", "S", "",
+		"Injected syscfg settings, key=value pairs separated by colon")
+
 	configCmd.AddCommand(configShowCmd)
 	AddTabCompleteFn(configShowCmd, func() []string {
 		return append(targetList(), unittestList()...)
@@ -798,6 +873,9 @@ func targetCfgCmdAll() []*cobra.Command {
 		Run:   targetConfigBriefCmd,
 	}
 
+	configBriefCmd.Flags().StringVarP(&util.InjectSyscfg, "syscfg", "S", "",
+		"Injected syscfg settings, key=value pairs separated by colon")
+
 	configCmd.AddCommand(configBriefCmd)
 	AddTabCompleteFn(configBriefCmd, func() []string {
 		return append(targetList(), unittestList()...)
@@ -810,6 +888,9 @@ func targetCfgCmdAll() []*cobra.Command {
 		Run:   targetConfigFlatCmd,
 	}
 
+	configFlatCmd.Flags().StringVarP(&util.InjectSyscfg, "syscfg", "S", "",
+		"Injected syscfg settings, key=value pairs separated by colon")
+
 	configCmd.AddCommand(configFlatCmd)
 	AddTabCompleteFn(configFlatCmd, func() []string {
 		return append(targetList(), unittestList()...)
@@ -905,6 +986,18 @@ func targetCfgCmdAll() []*cobra.Command {
 		return append(targetList(), unittestList()...)
 	})
 
+	sysinitGraphvizCmd := &cobra.Command{
+		Use:   "graphviz <target> [target...]",
+		Short: "Print a graph representation of target's sysinit configuration (DOT format)",
+		Long:  "Print a graph representation of target's sysinit configuration (DOT format)",
+		Run:   targetSysinitGraphvizCmd,
+	}
+
+	sysinitCmd.AddCommand(sysinitGraphvizCmd)
+	AddTabCompleteFn(sysinitGraphvizCmd, func() []string {
+		return append(targetList(), unittestList()...)
+	})
+
 	sysdownHelpText := "View a target's sysdown configuration"
 
 	sysdownCmd := &cobra.Command{
