@@ -93,6 +93,10 @@ type Downloader interface {
 	//
 	// If such a commit exists, it is returned.  Otherwise, "" is returned.
 	LatestRc(path string, base string) (string, error)
+
+	// Returns the branch that contains the YAML control files; this option
+	// allows implementers to override "master" as the main branch.
+	MainBranch() string
 }
 
 type Commit struct {
@@ -117,6 +121,7 @@ type GithubDownloader struct {
 	Server string
 	User   string
 	Repo   string
+	Branch string
 
 	// Login for private repos.
 	Login string
@@ -131,7 +136,8 @@ type GithubDownloader struct {
 
 type GitDownloader struct {
 	GenericDownloader
-	Url string
+	Url    string
+	Branch string
 }
 
 type LocalDownloader struct {
@@ -760,6 +766,9 @@ func (gd *GithubDownloader) Fetch(repoDir string) error {
 			gd.Repo)
 
 		cmd := []string{"fetch", "--tags"}
+		if util.ShallowCloneDepth > 0 {
+			cmd = append(cmd, "--depth", strconv.Itoa(util.ShallowCloneDepth))
+		}
 		_, err := gd.authenticatedCommand(repoDir, cmd)
 		return err
 	})
@@ -851,8 +860,7 @@ func (gd *GithubDownloader) setRemoteAuth(path string) error {
 }
 
 func (gd *GithubDownloader) Clone(commit string, dstPath string) error {
-	// Currently only the master branch is supported.
-	branch := "master"
+	branch := gd.MainBranch()
 
 	url, publicUrl := gd.remoteUrls()
 
@@ -871,10 +879,19 @@ func (gd *GithubDownloader) Clone(commit string, dstPath string) error {
 		"clone",
 		"-b",
 		branch,
-		url,
-		dstPath,
 	}
 
+	if util.ShallowCloneDepth > 0 {
+		cmd = append(cmd, "--depth", strconv.Itoa(util.ShallowCloneDepth))
+	} else if util.ShallowCloneDepth < 0 {
+		// default for clone, if not set by user, is to do shallow clone
+		cmd = append(cmd, "--depth", "1")
+		util.OneTimeWarning("Creating a shallow clone by default. " +
+			"To unshallow repository later use \"git fetch --unshallow\".")
+	}
+
+	cmd = append(cmd, url, dstPath)
+
 	if util.Verbosity >= util.VERBOSITY_VERBOSE {
 		err = util.ShellInteractiveCommand(cmd, nil, false)
 	} else {
@@ -908,13 +925,25 @@ func (gd *GithubDownloader) FixupOrigin(path string) error {
 	return gd.setOriginUrl(path, publicUrl)
 }
 
+func (gd *GithubDownloader) MainBranch() string {
+	if gd.Branch != "" {
+		return gd.Branch
+	} else {
+		return "master"
+	}
+}
+
 func NewGithubDownloader() *GithubDownloader {
 	return &GithubDownloader{}
 }
 
 func (gd *GitDownloader) Fetch(repoDir string) error {
 	return gd.cachedFetch(func() error {
-		_, err := executeGitCommand(repoDir, []string{"fetch", "--tags"}, true)
+		cmd := []string{"fetch", "--tags"}
+		if util.ShallowCloneDepth > 0 {
+			cmd = append(cmd, "--depth", strconv.Itoa(util.ShallowCloneDepth))
+		}
+		_, err := executeGitCommand(repoDir, cmd, true)
 		return err
 	})
 }
@@ -934,8 +963,7 @@ func (gd *GitDownloader) FetchFile(
 }
 
 func (gd *GitDownloader) Clone(commit string, dstPath string) error {
-	// Currently only the master branch is supported.
-	branch := "master"
+	branch := gd.MainBranch()
 
 	util.StatusMessage(util.VERBOSITY_DEFAULT,
 		"Downloading repository %s (commit: %s)\n", gd.Url, commit)
@@ -951,10 +979,19 @@ func (gd *GitDownloader) Clone(commit string, dstPath string) error {
 		"clone",
 		"-b",
 		branch,
-		gd.Url,
-		dstPath,
 	}
 
+	if util.ShallowCloneDepth > 0 {
+		cmd = append(cmd, "--depth", strconv.Itoa(util.ShallowCloneDepth))
+	} else if util.ShallowCloneDepth < 0 {
+		// default for clone, if not set by user, is to do shallow clone
+		cmd = append(cmd, "--depth", "1")
+		util.OneTimeWarning("Creating a shallow clone by default. " +
+			"To unshallow repository later use \"git fetch --unshallow\".")
+	}
+
+	cmd = append(cmd, gd.Url, dstPath)
+
 	if util.Verbosity >= util.VERBOSITY_VERBOSE {
 		err = util.ShellInteractiveCommand(cmd, nil, false)
 	} else {
@@ -985,6 +1022,14 @@ func (gd *GitDownloader) FixupOrigin(path string) error {
 	return setRemoteUrl(path, "origin", gd.Url, true)
 }
 
+func (gd *GitDownloader) MainBranch() string {
+	if gd.Branch != "" {
+		return gd.Branch
+	} else {
+		return "master"
+	}
+}
+
 func NewGitDownloader() *GitDownloader {
 	return &GitDownloader{}
 }
@@ -1005,7 +1050,7 @@ func (ld *LocalDownloader) FetchFile(
 
 func (ld *LocalDownloader) Fetch(path string) error {
 	os.RemoveAll(path)
-	return ld.Clone("master", path)
+	return ld.Clone(ld.MainBranch(), path)
 }
 
 func (ld *LocalDownloader) Checkout(path string, commit string) error {
@@ -1032,6 +1077,10 @@ func (ld *LocalDownloader) FixupOrigin(path string) error {
 	return nil
 }
 
+func (gd *LocalDownloader) MainBranch() string {
+	return "master"
+}
+
 func NewLocalDownloader() *LocalDownloader {
 	return &LocalDownloader{}
 }
@@ -1051,6 +1100,7 @@ func LoadDownloader(repoName string, repoVars map[string]string) (
 		gd.Server = repoVars["server"]
 		gd.User = repoVars["user"]
 		gd.Repo = repoVars["repo"]
+		gd.Branch = repoVars["branch"]
 
 		// The project.yml file can contain github access tokens and
 		// authentication credentials, but this file is probably world-readable
@@ -1080,6 +1130,7 @@ func LoadDownloader(repoName string, repoVars map[string]string) (
 	case "git":
 		gd := NewGitDownloader()
 		gd.Url = repoVars["url"]
+		gd.Branch = repoVars["branch"]
 		if gd.Url == "" {
 			return nil, loadError("repo \"%s\" missing required field \"url\"",
 				repoName)
