@@ -22,6 +22,7 @@ package newtutil
 import (
 	"fmt"
 	"io/ioutil"
+	"path/filepath"
 	"strconv"
 	"strings"
 
@@ -29,12 +30,12 @@ import (
 	"mynewt.apache.org/newt/util"
 )
 
-var NewtVersion = Version{1, 9, 0}
-var NewtVersionStr = "1.9.0"
+var NewtVersion = Version{1, 10, 0}
+var NewtVersionStr = "1.10.0"
 var NewtGitHash = "unknown"
 var NewtDate = "unknown"
 
-var NewtBlinkyTag string = "mynewt_1_9_0_tag"
+var NewtBlinkyTag string = "mynewt_1_10_0_tag"
 var NewtNumJobs int
 var NewtForce bool
 var NewtAsk bool
@@ -178,12 +179,17 @@ func MakeTempRepoDir() (string, error) {
 }
 
 func ProjRelPath(path string) string {
-	if strings.HasPrefix(path, "/") {
-		return path
+	if filepath.IsAbs(path) {
+		proj := interfaces.GetProject()
+		if proj != nil {
+			relPath, err := filepath.Rel(proj.Path(), path)
+			if err == nil {
+				path = relPath
+			}
+		}
 	}
 
-	proj := interfaces.GetProject()
-	return strings.TrimPrefix(path, proj.Path()+"/")
+	return path
 }
 
 func PrintNewtVersion() {
