@@ -60,8 +60,10 @@ type CfgSettingState int
 
 const (
 	CFG_SETTING_STATE_GOOD CfgSettingState = iota
+	CFG_SETTING_STATE_CONST
 	CFG_SETTING_STATE_DEPRECATED
 	CFG_SETTING_STATE_DEFUNCT
+	CFG_SETTING_STATE_EXPERIMENTAL
 )
 
 type CfgFlashConflictCode int
@@ -149,6 +151,12 @@ type Cfg struct {
 	// Use of defunct settings (error).
 	Defunct map[string]struct{}
 
+	// Use of const settings (error).
+	Consts map[string]struct{}
+
+	// Use of experimental settings (warning).
+	Experimental map[string]struct{}
+
 	// Unresolved value references
 	UnresolvedValueRefs map[string]struct{}
 }
@@ -166,6 +174,8 @@ func NewCfg() Cfg {
 		Redefines:           map[string]map[*pkg.LocalPackage]struct{}{},
 		Deprecated:          map[string]struct{}{},
 		Defunct:             map[string]struct{}{},
+		Consts:              map[string]struct{}{},
+		Experimental:        map[string]struct{}{},
 		UnresolvedValueRefs: map[string]struct{}{},
 	}
 }
@@ -212,6 +222,23 @@ func (cfg *Cfg) ExpandRef(val string) (string, string, error) {
 
 }
 
+func (cfg *Cfg) AddInjectedSettings() {
+	for _, setting := range strings.Split(util.InjectSyscfg, ":") {
+		kv := strings.SplitN(setting, "=", 2)
+		if len(kv) < 2 {
+			continue
+		}
+
+		k := kv[0]
+		v := kv[1]
+
+		if entry, ok := cfg.Settings[k]; ok {
+			entry.appendValue(nil, v)
+			cfg.Settings[k] = entry
+		}
+	}
+}
+
 func (cfg *Cfg) ResolveValueRefs() {
 	for k, entry := range cfg.Settings {
 		refName, val, err := cfg.ExpandRef(strings.TrimSpace(entry.Value))
@@ -328,9 +355,7 @@ func (entry *CfgEntry) ambiguities() []CfgPoint {
 		}
 
 		// If the two package have different priorities, there is no ambiguity.
-		if normalizePkgType(cur.Source.Type()) !=
-			normalizePkgType(next.Source.Type()) {
-
+		if findPkgPriority(cur.Source) != findPkgPriority(next.Source) {
 			break
 		}
 
@@ -380,10 +405,10 @@ func (entry *CfgEntry) priorityViolations() []CfgPriority {
 
 		// Overrides must come from a higher priority package, with one
 		// exception: a package can override its own setting.
-		curType := normalizePkgType(p.Source.Type())
-		defType := normalizePkgType(entry.PackageDef.Type())
+		curPrio := findPkgPriority(p.Source)
+		defPrio := findPkgPriority(entry.PackageDef)
 
-		if p.Source != entry.PackageDef && curType <= defType && entry.History[0].Value != "" {
+		if p.Source != entry.PackageDef && curPrio <= defPrio && entry.History[0].Value != "" {
 			priority := CfgPriority{
 				PackageDef:  entry.PackageDef,
 				PackageSrc:  p.Source,
@@ -432,14 +457,38 @@ func readSetting(name string, lpkg *pkg.LocalPackage,
 		entry.State = CFG_SETTING_STATE_DEFUNCT
 	} else if boolValue(vals["deprecated"]) {
 		entry.State = CFG_SETTING_STATE_DEPRECATED
+	} else if boolValue(vals["experimental"]) {
+		entry.State = CFG_SETTING_STATE_EXPERIMENTAL
 	} else {
 		entry.State = CFG_SETTING_STATE_GOOD
 	}
 
+	stateVal, stateExist := vals["state"]
+	if stateExist {
+		if entry.State != CFG_SETTING_STATE_GOOD {
+			util.OneTimeWarning("Setting %s has duplicated state definition, using \"state\".", name)
+		}
+
+		switch stringValue(stateVal) {
+		case "defunct":
+			entry.State = CFG_SETTING_STATE_DEFUNCT
+		case "deprecated":
+			entry.State = CFG_SETTING_STATE_DEPRECATED
+		case "experimental":
+			entry.State = CFG_SETTING_STATE_EXPERIMENTAL
+		default:
+			util.OneTimeWarning("Invalid \"state\" for setting %s, assuming regular state.", name)
+		}
+	}
+
 	// The value field for setting definition is required.
 	valueVal, valueExist := vals["value"]
 	if valueExist {
 		entry.Value = stringValue(valueVal)
+	} else if entry.State == CFG_SETTING_STATE_DEFUNCT {
+		// defunct settings do not need default value, we only care if
+		// value was set anywhere and emit an error then
+		entry.Value = ""
 	} else {
 		return entry, util.FmtNewtError(
 			"setting %s does not have required value field", name)
@@ -475,7 +524,14 @@ func readSetting(name string, lpkg *pkg.LocalPackage,
 	}
 
 	if vals["choices"] != nil {
-		choices := cast.ToStringSlice(vals["choices"])
+		var choices []string
+		switch vals["choices"].(type) {
+		default:
+			choices = cast.ToStringSlice(vals["choices"])
+		case string:
+			choices = strings.Split(vals["choices"].(string), ",")
+		}
+
 		entry.ValidChoices = choices
 
 		sort.Slice(choices, func(a, b int) bool {
@@ -544,19 +600,46 @@ func (cfg *Cfg) readDefsOnce(lpkg *pkg.LocalPackage,
 
 	if defs != nil {
 		for k, v := range defs {
-			vals, ok := v.(map[interface{}]interface{})
-			if !ok {
-				return util.FmtNewtError("Package \"%s\" contains invalid "+
-					"\"syscfg.defs\" map; expected full setting definition, "+
-					"but setting \"%s\" specifies a single value.  "+
-					"Did you mix up \"syscfg.defs\" and \"syscfg.vals\"?",
-					lpkg.FullName(), k)
-			}
+			var entry CfgEntry
+			var err error
 
-			entry, err := readSetting(k, lpkg, vals)
-			if err != nil {
-				return util.FmtNewtError("Config for package %s: %s",
-					lpkg.FullName(), err.Error())
+			switch v.(type) {
+			case map[interface{}]interface{}:
+				vals := v.(map[interface{}]interface{})
+				entry, err = readSetting(k, lpkg, vals)
+				if err != nil {
+					return util.FmtNewtError("Config for package %s: %s",
+						lpkg.FullName(), err.Error())
+				}
+			case int:
+				val := v.(int)
+				entry = CfgEntry{
+					Name:       k,
+					PackageDef: lpkg,
+					State:      CFG_SETTING_STATE_CONST,
+					Value:      stringValue(val),
+					History: []CfgPoint{{
+						Value:  stringValue(val),
+						Source: lpkg,
+					}},
+				}
+			case string:
+				val := v.(string)
+				entry = CfgEntry{
+					Name:       k,
+					PackageDef: lpkg,
+					State:      CFG_SETTING_STATE_CONST,
+					Value:      val,
+					History: []CfgPoint{{
+						Value:  val,
+						Source: lpkg,
+					}},
+				}
+			default:
+				return util.FmtNewtError("Package \"%s\" contains invalid "+
+					"\"syscfg.defs\" for \"%s\". It should be either full definition "+
+					"(configurable setting) or single value (const setting)",
+					lpkg.FullName(), k)
 			}
 
 			replace := true
@@ -652,6 +735,10 @@ func (cfg *Cfg) readValsOnce(lpkg *pkg.LocalPackage,
 			cfg.Deprecated[k] = struct{}{}
 		case CFG_SETTING_STATE_DEFUNCT:
 			cfg.Defunct[k] = struct{}{}
+		case CFG_SETTING_STATE_CONST:
+			cfg.Consts[k] = struct{}{}
+		case CFG_SETTING_STATE_EXPERIMENTAL:
+			cfg.Experimental[k] = struct{}{}
 		}
 	}
 
@@ -986,6 +1073,16 @@ func (cfg *Cfg) ErrorText() string {
 		}
 	}
 
+	// Overrides of const settings.
+	if len(cfg.Consts) > 0 {
+		str += "Override of consts settings detected:\n"
+		for name, _ := range cfg.Consts {
+			entry := cfg.Settings[name]
+			str += "    " + fmt.Sprintf("%s\n", name)
+			historyMap[name] = entry.History
+		}
+	}
+
 	// Unresolved value references
 	if len(cfg.UnresolvedValueRefs) > 0 {
 		str += "Unresolved value references:\n"
@@ -1050,10 +1147,39 @@ func (cfg *Cfg) DeprecatedWarning() []string {
 	return lines
 }
 
+func (cfg *Cfg) ExperimentalWarning() []string {
+	lines := []string{}
+
+	for k, _ := range cfg.Experimental {
+		entry, ok := cfg.Settings[k]
+		if !ok {
+			log.Errorf("Internal error; experimental setting \"%s\" not in cfg",
+				k)
+		}
+
+		point := mostRecentPoint(entry)
+		lines = append(lines,
+			fmt.Sprintf("Use of experimental setting %s in %s", k,
+				point.Source.FullName()))
+	}
+
+	return lines
+}
+
 func settingName(setting string) string {
 	return SYSCFG_PREFIX_SETTING + util.CIdentifier(setting)
 }
 
+func findPkgPriority(lpkg *pkg.LocalPackage) int {
+	if lpkg == nil {
+		// this is correct when doing lookup for a setting that was
+		// injected from command line, so let's assume this is always
+		// the case here
+		return int(pkg.PACKAGE_TYPE_TARGET)*pkg.PACKAGE_SUBPRIO_NUM + 1
+	}
+	return int(lpkg.Type())*pkg.PACKAGE_SUBPRIO_NUM + lpkg.SubPriority()
+}
+
 func normalizePkgType(typ interfaces.PackageType) interfaces.PackageType {
 	switch typ {
 	case pkg.PACKAGE_TYPE_TARGET:
@@ -1064,6 +1190,8 @@ func normalizePkgType(typ interfaces.PackageType) interfaces.PackageType {
 		return pkg.PACKAGE_TYPE_UNITTEST
 	case pkg.PACKAGE_TYPE_BSP:
 		return pkg.PACKAGE_TYPE_BSP
+	case pkg.PACKAGE_TYPE_CONFIG:
+		return pkg.PACKAGE_TYPE_CONFIG
 	default:
 		return pkg.PACKAGE_TYPE_LIB
 	}
@@ -1074,6 +1202,7 @@ func categorizePkgs(
 
 	pmap := map[interfaces.PackageType][]*pkg.LocalPackage{
 		pkg.PACKAGE_TYPE_TARGET:   []*pkg.LocalPackage{},
+		pkg.PACKAGE_TYPE_CONFIG:   []*pkg.LocalPackage{},
 		pkg.PACKAGE_TYPE_APP:      []*pkg.LocalPackage{},
 		pkg.PACKAGE_TYPE_UNITTEST: []*pkg.LocalPackage{},
 		pkg.PACKAGE_TYPE_BSP:      []*pkg.LocalPackage{},
@@ -1166,6 +1295,7 @@ func Read(lpkgs []*pkg.LocalPackage, apis []string,
 		pkg.PACKAGE_TYPE_BSP,
 		pkg.PACKAGE_TYPE_UNITTEST,
 		pkg.PACKAGE_TYPE_APP,
+		pkg.PACKAGE_TYPE_CONFIG,
 		pkg.PACKAGE_TYPE_TARGET,
 	} {
 		if err := cfg.readDefsForPkgType(lpkgMap[ptype], settings); err != nil {
@@ -1178,6 +1308,7 @@ func Read(lpkgs []*pkg.LocalPackage, apis []string,
 		pkg.PACKAGE_TYPE_BSP,
 		pkg.PACKAGE_TYPE_UNITTEST,
 		pkg.PACKAGE_TYPE_APP,
+		pkg.PACKAGE_TYPE_CONFIG,
 		pkg.PACKAGE_TYPE_TARGET,
 	} {
 		if err := cfg.readValsForPkgType(lpkgMap[ptype], settings); err != nil {
@@ -1309,7 +1440,11 @@ func writeDefine(key string, value string, w io.Writer) {
 		fmt.Fprintf(w, "#undef %s\n", key)
 	} else {
 		fmt.Fprintf(w, "#ifndef %s\n", key)
-		fmt.Fprintf(w, "#define %s (%s)\n", key, value)
+		if strings.HasPrefix(value, "\"") && strings.HasSuffix(value, "\"") {
+			fmt.Fprintf(w, "#define %s %s\n", key, value)
+		} else {
+			fmt.Fprintf(w, "#define %s (%s)\n", key, value)
+		}
 		fmt.Fprintf(w, "#endif\n")
 	}
 }
@@ -1403,17 +1538,17 @@ func writeReposInfo(w io.Writer) {
 		r := project.GetProject().Repos()[n]
 
 		if !r.CheckExists() {
-			continue;
+			continue
 		}
 
 		commitHash, err := r.CurrentHash()
 		if err != nil {
-			continue;
+			continue
 		}
 
 		fmt.Fprintf(w, "/*** Repository @%s info */\n", r.Name())
 
-		k := fmt.Sprintf("%s", settingName("REPO_HASH_" + util.CIdentifier(strings.ToUpper(r.Name()))))
+		k := fmt.Sprintf("%s", settingName("REPO_HASH_"+util.CIdentifier(strings.ToUpper(r.Name()))))
 		v := fmt.Sprintf("\"%s", commitHash)
 
 		dirty, err := r.DirtyState()
@@ -1434,10 +1569,10 @@ func writeReposInfo(w io.Writer) {
 		if err != nil || ver == nil {
 			v = "0.0.0"
 		} else {
-			v = ver.String();
+			v = ver.String()
 		}
 
-		k = fmt.Sprintf("%s", settingName("REPO_VERSION_" + util.CIdentifier(strings.ToUpper(r.Name()))))
+		k = fmt.Sprintf("%s", settingName("REPO_VERSION_"+util.CIdentifier(strings.ToUpper(r.Name()))))
 		v = fmt.Sprintf("\"%s\"", v)
 
 		writeDefine(k, v, w)
@@ -1454,8 +1589,10 @@ func write(cfg Cfg, w io.Writer) {
 	writeCheckMacros(w)
 	fmt.Fprintf(w, "\n")
 
-	writeReposInfo(w)
-	fmt.Fprintf(w, "\n")
+	if !util.SkipSyscfgRepoHash {
+		writeReposInfo(w)
+		fmt.Fprintf(w, "\n")
+	}
 
 	writeSettings(cfg, w)
 	fmt.Fprintf(w, "\n")
